@@ -1,27 +1,24 @@
-# Copyright 2012-2013 Francesco Napolitano, franapoli@gmail.com
-#
-# This file is part of pyleaf.
-#
-#     pyleaf is free software; you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation; either version 2 of the License, or
-#     (at your option) any later version.
-#
-#     pyleaf is distributed in the hope that it will be useful, but
-#     WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public
-#     License along with pyleaf; if not, write to the Free Software
-#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-#     02110-1301 USA
+# The MIT License (MIT)
 
-"""
-Created on Fri Oct 22 15:59:38 2010
+# Copyright (c) 2012-2013 Francesco Napolitano, franapoli@gmail.com
 
-@author: ciccio
-"""
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
 
 import os
 import pickle
@@ -36,8 +33,24 @@ import sys
 class protocol():
     """Leaf Protocol
     
-    Manages all protocol resources and provides user interface through Python command line.
-    Protocols are created by the prj class: use it to obtain a protocol object.
+    Manages all protocol resources and provides user interface through
+    Python command line. Protocols are created by the prj class: use
+    it to obtain a protocol object.
+
+    Pipeline nodes are executed through the run and provide
+    methods. These methods will return the corresponding resource
+    immediately if available in memory, load it from the disk if
+    previously dumped, or run the appropriate nodes according to the
+    protocol.
+
+    If dumping is ON (method dumpOn) automatic caching is active and
+    the output of all nodes is stored in the leaf_MODULENAME directory
+    as the files NODENAME.res. The files NODENAME.mod within the same
+    directory are used to store nodes source code. All the data in
+    these files are wrapped in leaf.rrc.resource objects which dump
+    themselves through the pickle.dump method. Other files in the
+    directory are produced by the leaf.prj.project class.
+    
     """    
 
     def __init__(self, graph, mods, folder, doc):
@@ -103,19 +116,25 @@ class protocol():
             self.untrust(mod)
 
     def listinputs(self, node):
+        """Lists the inputs for a node."""
         return self._getGraph().getInNodes(node)
 
-    def listoutputs(self, node): 
+    def listoutputs(self, node):
+        """Lists the outputs for a node."""
         return self._getGraph().getOutNodes(node)       
 
+    def resetall(self):
+        """Clears and undumps all resources"""
+        for res in self._getResNames():
+            self.reset(res)
 
     def clearall(self):
-        """Clears all dumped resources"""
+        """Clears all resources from RAM"""
         for res in self._getResNames():
             self.clear(res)
 
     def undumpall(self):
-        """Deletes all dumped resources"""
+        """Deletes all resources from the disk"""
         for res in self._getResNames():
             self.undump(res)
 
@@ -125,13 +144,16 @@ class protocol():
 
     def getinputs(self, mod):
         """Collects all input resources that are input to the given filter and returns a copy of them in a list."""
+        if type(mod) != str:
+            mod = mod.__name__
+
         innodes = self._getInNodes(mod)
         ids = [self._getGraph().getAttrib(_node, 'id') for _node in innodes]
         nodeparams = [copy.deepcopy(self._getResource(innode).getValue()) for innode in innodes]
         nodeparams=[one for (one,two) in sorted(zip(nodeparams, ids), key = lambda x:x[1])]      
         return tuple(nodeparams)
 
-    def run(self, serial = False):
+    def run(self, parallel = False):
         """Provides all leaf (final) resources."""
         res = None
         allok = True
@@ -144,7 +166,7 @@ class protocol():
         if allok:
             dbgstr('Nothing to be done. Zzz...')
         else:
-            res = self.provide(l, serial)
+            res = self.provide(l, parallel)
         return res
     
     def untrust(self, nodename):
@@ -443,7 +465,7 @@ class protocol():
     def _prettyPrint(self, x):
         return str(x).strip('[]').replace("'","")
 
-    def provide(self, res, serial=False):
+    def provide(self, res, parallel=False):
         t = time.time()
 
         if not type(res) == list:
@@ -472,13 +494,11 @@ class protocol():
 
         while True:
             canrun = self._findRunnable(needbuild)
-            if len(canrun) > 1:
+            if len(canrun) > 1 and parallel:
                 dbgstr('The following nodes can run in parallel: '+
                        self._prettyPrint(canrun))
-                if serial:
-                    dbgstr('But serial processing was requested!')
 
-            self._runNodes(canrun, serial)
+            self._runNodes(canrun, parallel)
             needbuild=[x for x in needbuild if not x in canrun]
 
             if len(canrun)==0:
@@ -496,8 +516,8 @@ class protocol():
         else:
             return builtres[0]
 
-    def _runNodes(self, nodes, serial = False):
-        if not serial:
+    def _runNodes(self, nodes, parallel = False):
+        if parallel and len(nodes)>1:
             from multiprocessing import Process, Queue
             results = Queue()
             tasks = list()
@@ -517,17 +537,12 @@ class protocol():
                 self._newResource(resname, taskres[1], t)
         else:
             for node in nodes:
-                # the following mimics the parallel code to avoid
-                # changing code, but should be polished
-                from multiprocessing import Process, Queue
-                results = Queue()
                 nodeparams = self._getNodePar(node)
-                self._callModPar(node, nodeparams, results)
-                taskres = results.get()
-                resname = self._buildResName(taskres[0], None, taskres[1])
-                dbgstr('Requesting add resource: ' + taskres[0], 2)
+                taskres = self._callMod(node, nodeparams)
+                resname = self._buildResName(taskres[1], None, taskres[0])
+                dbgstr('Requesting add resource: ' + resname, 2)
                 t=taskres[2]
-                self._newResource(resname, taskres[1], t)
+                self._newResource(resname, taskres[0], t)
 
     def time(self, node):
         if type(node) != str:
@@ -858,9 +873,9 @@ class protocol():
         newresname = self._buildResName(node, None, newres)
         dbgstr('Requesting add resource: ' + node, 2)
         t = time.time() - t
-        self._newResource(newresname, newres, t)
+        #self._newResource(newresname, newres, t)
             
-        return newres, newresname
+        return newres, newresname, t
 
     def _callModPar(self, node, nodeparams, queue):
            
